Package: NanoStringDiff
Type: Package
Title: Differential Expression Analysis of NanoString nCounter Data
Version: 1.41.0
Date: 2018-01-24
Imports: matrixStats, methods, Rcpp
LinkingTo: Rcpp
Depends: Biobase
Suggests: testthat, BiocStyle
Author: hong wang <hong.wang@uky.edu>, tingting zhai <tingting.zhai@uky.edu>, chi wang <chi.wang@uky.edu>
Maintainer: tingting zhai <tingting.zhai@uky.edu>,hong wang <hong.wang@uky.edu>
Description: This Package utilizes a generalized linear model(GLM) of
        the negative binomial family to characterize count data and
        allows for multi-factor design. NanoStrongDiff incorporate size
        factors, calculated from positive controls and housekeeping
        controls, and background level, obtained from negative
        controls, in the model framework so that all the normalization
        information provided by NanoString nCounter Analyzer is fully
        utilized.
License: GPL
biocViews: DifferentialExpression, Normalization
NeedsCompilation: yes
Packaged: 2025-10-30 22:10:24 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/NanoStringDiff
git_branch: devel
git_last_commit: b94a57b
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30

import( Biobase )
import( methods )
import(Rcpp)
importFrom( "matrixStats","rowMins","rowMaxs")
importFrom(Rcpp, evalCpp)
exportClasses("NanoStringSet") 
exportMethods(
              positiveFactor, "positiveFactor<-",
              negativeFactor, "negativeFactor<-",
              housekeepingFactor, "housekeepingFactor<-",
              positiveControl, "positiveControl<-",
              negativeControl, "negativeControl<-",
              housekeepingControl, "housekeepingControl<-"
              )

export(
       createNanoStringSet,
       createNanoStringSetFromCsv,
       estNormalizationFactors, 
       glm.LRT,
       NanoStringDataNormalization,
       PlotsPositiveHousekeeping
)

exportPattern("^[[:alpha:]]+")
useDynLib(NanoStringDiff)

rnegbinom <- function(n, mu = 1, phi = 0.01) {
    rpois(n, rgamma(n, shape = 1/phi, scale = mu * phi))
}


est.dispersion <- function(Y, Y_nph, lamda_i, c, d) {
    
    nsamples = ncol(Y)
    
    mu = rowMeans(Y_nph)
    muY = sweep(matrix(rep(mu, nsamples), ncol = nsamples), 2, c * d, FUN = "*")
    
    
    get.phihat <- function(dat) {
        y = dat[1:nsamples]
        Ey = dat[nsamples + (1:nsamples)]
        
        obj = function(phi) {
            alpha = 1/phi
            tmp1 = 1/(1 + Ey * phi)
            tmp2 = 1 - tmp1
            tmp2[tmp2==0] = 1e-08
            tmp3.p = fun5(cbind(matrix(y, ncol = 1), matrix(lamda_i, ncol = 1), 
                                matrix(tmp2, ncol = 1),rep(alpha,length(y))))
            -tmp3.p
        }
        return(optimize(obj, interval = c(1e-08, 1e+08))$minimum)
    }
    
    
    phi.g = apply(cbind(matrix(Y, ncol = nsamples), 
                        matrix(muY, ncol = nsamples)), 1, get.phihat)
    
    lphi = log(phi.g)
    
    list(phi = phi.g, eta = lphi)
}


compute.baseSigma <- function(phi0, Y, muY, nsamples) {
    set.seed(123)
    nsim = 5
    res = rep(0, nsim)
    c = rep(1, nsamples)
    d = rep(1, nsamples)
    lamda = rep(20, nsamples)
    for (i in 1:nsim) {
        Ysim <- matrix(rnegbinom(length(Y), muY, phi = phi0), ncol = nsamples)
        ## assume 20 is background noise
        lag.sim = est.dispersion(Ysim + 20, Ysim, lamda, c, d)$eta
        sigma = IQR(lag.sim, na.rm = TRUE)/1.349
        res[i] = sigma^2
    }
    mean(res)
} 

estNormalizationFactors <- function(NanoStringData) {
    
    if (!is(NanoStringData, "NanoStringSet")) 
        stop("Input must be an object of NanoStringSet class!")
    
    
    
    pos = positiveControl(NanoStringData)
    pos[pos <= 0] = 1
    neg = negativeControl(NanoStringData)
    x = c(128, 32, 8, 2, 0.5, 0.125, rep(0, nrow(neg)))
    Y = rbind(pos, neg)
    n = ncol(Y)
    lamda_i = rep(0, n)
    c = rep(0, n)
    for (i in 1:n) {
        
        model = glm(Y[, i] ~ x, family = poisson(link = identity))
        lamda_i[i] = round(model$coeff[1])
        
        c[i] = model$coeff[2]
    }
    
    coeff = c  # real coefficients
    c = c/mean(c)  # size factors
    
    names(lamda_i) = colnames(pos)
    names(c) = colnames(pos)
    lamda_i[lamda_i == 0] = 1
    
    
    
    ## estimate housekeeping factors
    house1 = sweep(housekeepingControl(NanoStringData), 2, lamda_i, FUN = "-")
    house1[house1 <= 0] = 1
    house2 = sweep(house1, 2, c, FUN = "/")
    temp2 = sweep(house2, 1, rowMeans(house2), FUN = "/")
    d = apply(temp2, 2, median)  #housekeeping control factors
    
    positiveFactor(NanoStringData) = c
    negativeFactor(NanoStringData) = lamda_i
    housekeepingFactor(NanoStringData) = d
    NanoStringData
} 

## generate count matrix
endogenous = matrix(c(100,100,100,100,100,100,50,50,300,300,100,100),
                    3,4, byrow = TRUE)
sample.names = paste0("Sample",1:4)
colnames(endogenous) = sample.names
positive = matrix(c(128,32,8,2,0.5,0.125)*80,6,4)
colnames(positive) = paste("Sample",1:4)
negative = matrix(10,8,4)
colnames(negative) = sample.names
housekeeping = matrix(100,3,4)
colnames(housekeeping) = sample.names

## generate phenotype data
designs = data.frame(group=c("Control","Control","Treatment","Treatment"),
                     gender=c("Male","Female","Female","Male"),
                     age=c(20,40,39,37))
## input data to create a "NanoStringSet" object
NanoStringData = createNanoStringSet(endogenous,positive,
                                  negative,housekeeping,designs)
NanoStringData = estNormalizationFactors(NanoStringData)
pheno=pData(NanoStringData)
group=pheno$group
design.full=model.matrix(~0+group)

result = glm.LRT(NanoStringData, design.full, contrast = c(1, -1))


lfc1 = result$table[1,1]
lfc2 = result$table[2,1]
lfc3 = result$table[3,1]
expect_equal(lfc1, log2(1), tolerance=.01)
expect_equal(lfc2, log2(2), tolerance=.2)
expect_equal(lfc3, log2(3), tolerance=.2)



result = glm.LRT(NanoStringData, design.full, contrast = c(-1, 1))

lfc1 = result$table[1,1]
lfc2 = result$table[2,1]
lfc3 = result$table[3,1]
expect_equal(lfc1, -log2(1), tolerance=.01)
expect_equal(lfc2, -log2(2), tolerance=.2)
expect_equal(lfc3, -log2(3), tolerance=.2)

## generate count matrix
endogenous = matrix(rpois(100,50),25,4)
sample.names = paste0("Sample",1:4)
colnames(endogenous) = sample.names
positive = matrix(c(128,32,8,2,0.5,0.125)*80,6,4)
colnames(positive) = paste("Sample",1:4)
negative = matrix(10,8,4)
colnames(negative) = sample.names
housekeeping = matrix(100,3,4)
colnames(housekeeping) = sample.names

## generate phenotype data
designs = data.frame(group=c("Control","Control","Treatment","Treatment"),
                   gender=c("Male","Female","Female","Male"),
                   age=c(20,40,39,37))
## input data to create a "NanoStringSet" object
NanoStringData = createNanoStringSet(endogenous,positive,
                                negative,housekeeping,designs)
NanoStringData = estNormalizationFactors(NanoStringData)




posFactor = matrix(1,1,4)
houFactor = matrix(1,1,4)

expect_true(all(positiveFactor(NanoStringData) == posFactor))
expect_true(all(housekeepingFactor(NanoStringData) == houFactor))

PlotsPositiveHousekeeping<- function(path=path, header=TRUE){
  # ------------------------------------ #
  # path: points out the directory in which your csv.file is located #
  # header: a logical value(TRUE or FALSE) indicating whether the file contains the names of the variables as its first line. #
  # designs: data frame for pheno type data storage #
  # ------------------------------------ #
  data = read.table(path, header = header, sep = ",")
  
  if (is.null(data)) {
      stop("There is no counts data")
  }
  
  selectcol = !(names(data) %in% c("Code.Class", "Name", "Accession"))
  
  ## remove NA from data set
  counts = data[,selectcol]
  counts = as.matrix(counts)
  id = which(is.na(rowSums(counts)))
  if (length(id) > 0) {
      data = data[-id, ]
  }
  
  code.class = tolower(data[, c("Code.Class")])
  name = data[, c("Name")]
  accession = data[, c("Accession")]
  
  counts = data[,selectcol]
  counts = as.matrix(counts)
  rownames(counts) = name
  
  pos.id = grep("positive", code.class, fixed = TRUE)
  neg.id = grep("negative", code.class, fixed = TRUE)
  house.id = grep("housekeeping", code.class, fixed = TRUE)
  spikein.id = grep("spikein", code.class, fixed = TRUE)
  
  positive = counts[pos.id, ]
  negative = counts[neg.id, ]
  housekeeping = counts[house.id, ]
  house.adjust = counts[house.id, ]
  
  mean.neg<-colMeans(negative)
  
  pos = positive
  pos[pos <= 0] = 1
  neg = negative
  x = c(128, 32, 8, 2, 0.5, 0.125, rep(0, nrow(neg)))
  Y = rbind(pos, neg)
  n = ncol(Y)
  c = rep(0, n)
  for (i in 1:n) {
      
      model = glm(Y[, i] ~ x, family = poisson(link = identity))
      c[i] = model$coeff[2]
  }
  
  coeff = c  # real coefficients
  c = c/mean(c)  # size factors
  
  for (i in 1:nrow(housekeeping)){
      housekeeping[i,]<-(housekeeping[i,]-mean.neg)/c^(1/2)
  }
  
  nhouse<-nrow(housekeeping)
  varhouse<-rowVars(housekeeping)^(1/2)/rowMeans(housekeeping)
  con=c(128,32,8,2,0.5,0.125)
  par(mfrow=c(1,2),mar=c(5,6,2,1)+0.1, oma=c(2,2,2,2)+0.1)
  plot(con,rowMeans(positive),xlab="Concentration",ylab="Positive Counts",
  col = "blue")
  abline(lm(rowMeans(positive) ~ con))
  plot(c(1:nhouse),varhouse,xaxt='n',xlab="",ylab="Coefficient of Variation",
  col = "blue", xlim =c(1,nhouse+1), ylim =c(min(varhouse)*0.9,max(varhouse)*1.1))
  title(xlab="Housekeeping Genes", line=1.5)
  textxy(c(1:nhouse), varhouse, labs=rownames(housekeeping), cex = 1, m = c(0, 0), offset = 0.65)
} # end function #

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

fun5 <- function(yy_g) {
    .Call('_NanoStringDiff_fun5', PACKAGE = 'NanoStringDiff', yy_g)
}

setClass("NanoStringSet", 
         contains = "ExpressionSet", 
         representation = representation(
             positiveFactor = "numeric", 
             negativeFactor = "numeric", 
             housekeepingFactor = "numeric", 
             positiveControl = "matrix", 
             negativeControl = "matrix", 
             housekeepingControl = "matrix"))
### normalization factor positiveFactor
setGeneric("positiveFactor", function(object) standardGeneric("positiveFactor"))
setGeneric("positiveFactor<-", function(object, value) standardGeneric("positiveFactor<-"))
setMethod("positiveFactor", signature(object = "NanoStringSet"), function(object) {
    object@positiveFactor
    
})
setReplaceMethod("positiveFactor", signature(object = "NanoStringSet", value = "numeric"), 
                 function(object, value) {
                     n = ncol(exprs(object))
                     if (length(value) != n) 
                         stop("wrong length for positive factor vector!")
                     object@positiveFactor <- value
                     object
                 })
### negativeFactor
setGeneric("negativeFactor", function(object) standardGeneric("negativeFactor"))
setGeneric("negativeFactor<-", function(object, value) standardGeneric("negativeFactor<-"))
setMethod("negativeFactor", signature(object = "NanoStringSet"), function(object) {
    object@negativeFactor
})
setReplaceMethod("negativeFactor", signature(object = "NanoStringSet", value = "numeric"), 
                 function(object, value) {
                     n = ncol(exprs(object))
                     if (length(value) != n) 
                         stop("wrong length for negative factor vector!")
                     object@negativeFactor <- value
                     object
                 })
### housekeepingFactor
setGeneric("housekeepingFactor", function(object) standardGeneric("housekeepingFactor"))
setGeneric("housekeepingFactor<-", function(object, value) standardGeneric("housekeepingFactor<-"))
setMethod("housekeepingFactor", signature(object = "NanoStringSet"), function(object) {
    object@housekeepingFactor
})
setReplaceMethod("housekeepingFactor", signature(object = "NanoStringSet", value = "numeric"), 
                 function(object, value) {
                     n = ncol(exprs(object))
                     if (length(value) != n) 
                         stop("wrong length for housekeeping factor vector!")
                     object@housekeepingFactor <- value
                     object
                 })
################## control genes positive control genes
setGeneric("positiveControl", function(object) standardGeneric("positiveControl"))
setGeneric("positiveControl<-", function(object, value) standardGeneric("positiveControl<-"))
setMethod("positiveControl", signature(object = "NanoStringSet"), function(object) {
    object@positiveControl
})
setReplaceMethod("positiveControl", signature(object = "NanoStringSet", value = "matrix"), 
                 function(object, value) {
                     object@positiveControl <- value
                     object
                 })
### negative control genes
setGeneric("negativeControl", function(object) standardGeneric("negativeControl"))
setGeneric("negativeControl<-", function(object, value) standardGeneric("negativeControl<-"))
setMethod("negativeControl", signature(object = "NanoStringSet"), function(object) {
    object@negativeControl
})
setReplaceMethod("negativeControl", signature(object = "NanoStringSet", value = "matrix"), 
                 function(object, value) {
                     object@negativeControl <- value
                     object
                 })
### housekeeping control genes
setGeneric("housekeepingControl", function(object) standardGeneric("housekeepingControl"))
setGeneric("housekeepingControl<-", function(object, value) standardGeneric("housekeepingControl<-"))
setMethod("housekeepingControl", signature(object = "NanoStringSet"), function(object) {
    object@housekeepingControl
})
setReplaceMethod("housekeepingControl", signature(object = "NanoStringSet", value = "matrix"), 
                 function(object, value) {
                     object@housekeepingControl <- value
                     object
                 })
################# read data#####
createNanoStringSet <- function(endogenous, positiveControl, 
                             negativeControl, housekeepingControl, designs) {
    ## work on input design matrix. Carefully about the input design.
    if (is(designs, "vector")) {
        designs <- as.data.frame(designs)
        colnames(designs) = "designs"
    } else if (is(designs, "matrix")) {
        ## multiple factor design still single factor
        if (ncol(designs) == 1) {
            designs <- as.data.frame(designs)
            colnames(designs) = "designs"
        } else {
            ## input is a data frame
            designs <- as.data.frame(designs)
        }
    }
    
    rownames(designs) = colnames(endogenous)
    
    
    if (is(designs, "data.frame") || is(designs, "AnnotatedDataFrame")) {
        stopifnot(nrow(designs) == ncol(endogenous))
        designs <- as(designs, "AnnotatedDataFrame")
    }
    res <- new("NanoStringSet", exprs = endogenous, phenoData = designs, 
               positiveControl = positiveControl, 
               negativeControl = negativeControl, 
               housekeepingControl = housekeepingControl)
    
    res
    
}
createNanoStringSetFromCsv <- function(path, header = TRUE, designs) {
    
    ## read data from csv file
    
    data = read.table(path, header = header, sep = ",")
    
    if (is.null(data)) {
        stop("There is no counts data")
    }
    

    selectcol = !(names(data) %in% c("Code.Class", "Name", "Accession"))
    
    ## remove NA from data set
    counts = data[,selectcol]
    counts = as.matrix(counts)
    id = which(is.na(rowSums(counts)))
    if (length(id) > 0) {
        data = data[-id, ]
    }
    
    
    code.class = tolower(data[, c("Code.Class")])
    name = data[, c("Name")]
    accession = data[, c("Accession")]
    
    
    counts = data[,selectcol]
    counts = as.matrix(counts)
    rownames(counts) = name
    
    pos.id = grep("positive", code.class, fixed = TRUE)
    neg.id = grep("negative", code.class, fixed = TRUE)
    house.id = grep("housekeeping", code.class, fixed = TRUE)
    spikein.id = grep("spikein", code.class, fixed = TRUE)
    
    
    positive = counts[pos.id, ]
    negative = counts[neg.id, ]
    housekeeping = counts[house.id, ]
    spikein = counts[spikein.id, ]
    endogenous = counts[-c(pos.id, neg.id, house.id, spikein.id), ]
    
    
    cat(paste(" There are", ncol(counts), "samples imported; \n There are ", 
              nrow(counts), "genes imported with:"))
    
    
    print(table(code.class))
    
    
    
    
    ## work on input design matrix. Carefully about the input design.
    if (is(designs, "vector")) {
        designs <- as.data.frame(designs)
        colnames(designs) = "designs"
    } else if (is(designs, "matrix")) {
        ## multiple factor design still single factor
        if (ncol(designs) == 1) {
            designs <- as.data.frame(designs)
            colnames(designs) = "designs"
        } else {
            ## input is a data frame
            designs <- as.data.frame(designs)
        }
    }
    
    
    rownames(designs) = colnames(counts)
    
    
    if (is(designs, "data.frame") || is(designs, "AnnotatedDataFrame")) {
        stopifnot(nrow(designs) == ncol(counts))
        designs <- as(designs, "AnnotatedDataFrame")
    }
    
    
    
    rownames(designs) = colnames(counts)
    
    
    
    res <- new("NanoStringSet", exprs = endogenous, phenoData = designs, 
               positiveControl = positive, 
               negativeControl = negative, 
               housekeepingControl = housekeeping)
    
    res
    
} 

## generate count matrix
endogenous = matrix(rpois(100,50),25,4)
sample.names = paste0("Sample",1:4)
colnames(endogenous) = sample.names
positive = matrix(rpois(24,c(128,32,8,2,0.5,0.125)*30),6,4)
colnames(positive) = paste("Sample",1:4)
negative = matrix(rpois(32,10),8,4)
colnames(negative) = sample.names
housekeeping = matrix(rpois(12,100),3,4)
colnames(housekeeping) = sample.names

## generate phenotype data
designs = data.frame(group=c("Control","Control","Treatment","Treatment"),
                   gender=c("Male","Female","Female","Male"),
                   age=c(20,40,39,37))
## input data to create a "NanoStringSet" object
NanoStringData = createNanoStringSet(endogenous,positive,
                                 negative,housekeeping,designs)

expect_true(all(exprs(NanoStringData) == endogenous))
expect_true(all(positiveControl(NanoStringData) == positive))
expect_true(all(negativeControl(NanoStringData) == negative))
expect_true(all(housekeepingControl(NanoStringData) == housekeeping))
expect_true(all(colnames(exprs(NanoStringData)) == sample.names))

NanoStringDataNormalization <- function(path=path, header=TRUE, designs){
  # ------------------------------------ #
  # path: points out the directory in which your csv.file is located #
  # header: a logical value(TRUE or FALSE) indicating whether the file contains the names of the variables as its first line. #
  # designs: data frame for pheno type data storage #
  # ------------------------------------ #
  NanoStringData <- createNanoStringSetFromCsv(path, header, designs) # Create a NanoStringSet needed in NanoStringDiff from csv file #
  NanoStringData <- estNormalizationFactors(NanoStringData) # get normalization factors #
  
  PosFactor <- positiveFactor(NanoStringData) # get positive size factor #
  HouseFactor <- housekeepingFactor(NanoStringData) # get housekeeping size factor #
  NegFactor <- negativeFactor(NanoStringData) # get background niose #
  RawData <- exprs(NanoStringData) # get raw data #
  
  NormalizedData <- round((RawData - NegFactor)/(PosFactor * HouseFactor)) # normalization #
  NormalizedData[NormalizedData < 0] <- 0 # set negative value as 0 #
  
  ColName <- colnames(NormalizedData)
  colnames(NormalizedData) <- paste("Normalized", ColName, sep = "") # change column for normalized data #
  
  return(list(RawData = RawData, NormalizedData = NormalizedData))
} # end function #

glm.LRT <- function(NanoStringData, design.full, Beta = ncol(design.full), contrast = NULL) {
    
    c = positiveFactor(NanoStringData)
    d = housekeepingFactor(NanoStringData)
    k = c * d
    lamda_i = negativeFactor(NanoStringData)
    
    
    if (length(k) == 0) {
        stop("Before calling function glm.LRT, should get normalization factors \n             
             first using function estNormalizationFactors")
    }
    
    
    Y = exprs(NanoStringData)
    Y_n = sweep(Y, 2, lamda_i, FUN = "-")
    Y_nph = sweep(Y_n, 2, k, FUN = "/")
    Y_nph[Y_nph <= 0] = 0.1
    
    X.full = design.full
    nsamples = ncol(Y)
    Beta.names = colnames(design.full)
    
    
    result.full = glmfit.full(NanoStringData, design.full)
    Beta.full = result.full$Beta.full
    U.full = result.full$mean.full
    phi.hat = result.full$dispersion
    df.full = result.full$df.full
    m0 = result.full$m0
    sigma = result.full$sigma
    
    V.full = sweep(U.full, 2, k, FUN = "*")
    
    
    ## Make reduced design matrix.
    ## Here we borrow the idea from paskage edgeR to make reduced design matrix
    
    
    
    
    if (is.null(contrast)) {
        if (length(Beta) > 1) 
            Beta = unique(Beta)
        
        if (is.character(Beta)) {
            check.Beta = Beta %in% Beta.names
            if (any(!check.Beta)) 
                stop("The name(s) of Beta arguments do not match the \n
                     name(s) of the design matrix.")
            Beta = match(Beta, Beta.names)
        }
        
        logFC = Beta.full[, Beta, drop = FALSE]/log(2)
    } else {
        contrast = as.matrix(contrast)
        qrc = qr(contrast)
        ncontrasts = qrc$rank
        
        if (ncontrasts == 0) 
            stop("Need at least one none zero contrast")
        
        Beta = 1:ncontrasts
        
        if (ncontrasts < ncol(contrast)) 
            contrast = contrast[, qrc$pivot[Beta]]
        
        logFC = drop((Beta.full %*% contrast)/log(2))
        
        
        Dvec = rep.int(1, nsamples)
        Dvec[Beta] = diag(qrc$qr)[Beta]
        Q = qr.Q(qrc, complete = TRUE, Dvec = Dvec)
        design.full = design.full %*% Q
    }
    
    
    design.reduce = design.full[, -Beta, drop = FALSE]
    
    
    if (ncol(design.reduce) == 1) {
        
        result.reduce = glmfit.OneGroup(NanoStringData, m0, sigma, phi.hat)
        
    } else {
        
        result.reduce = glmfit.reduce(NanoStringData, design.reduce, m0, sigma, phi.hat)
        
    }
    
    
    Beta.reduce = result.reduce$Beta.reduce
    U.reduce = result.reduce$mean.reduce
    df.reduce = result.reduce$df.reduce
    
    V.reduce = sweep(U.reduce, 2, k, FUN = "*")
    
    
    get.loglikelihood <- function(dat) {
        
        y = dat[1:nsamples]
        Ey = dat[nsamples + (1:nsamples)]
        phi = dat[2 * nsamples + 1]
        
        
        
        alpha = 1/phi
        tmp1 = 1/(1 + Ey * phi)
        tmp2 = 1 - tmp1
        tmp2[tmp2==0] = 1e-08
        
        
        item1 = function(yy) {
            y_gi = yy[1]
            lamda_gi = yy[2]
            tmp2_gi = yy[3]
            t = c(0:y_gi)
            tmp33.t = exp(lgamma(t + alpha) + (y_gi - t) * log(lamda_gi) + t * log(tmp2_gi) - 
                              lfactorial(t) - lfactorial(y_gi - t))
            tmp33.tt = log(max(sum(tmp33.t), 1e-08))
        }
        
        
        tmp3 = apply(cbind(matrix(y, ncol = 1), matrix(lamda_i, ncol = 1), matrix(tmp2, 
                                                                                  ncol = 1)), 1, item1)
        
        sum(tmp3) - nsamples * lgamma(alpha) + alpha * sum(log(tmp1)) - sum(lamda_i)
    }
    
    
    ## compute likelihood under null
    tmpl1 = cbind(Y, V.reduce, phi.hat)
    l0 = apply(tmpl1, 1, get.loglikelihood)
    
    
    ## compute likelihood under alternative
    
    tmpl2 = cbind(Y, V.full, phi.hat)
    la = apply(tmpl2, 1, get.loglikelihood)
    
    
    lr = -2 * (l0 - la)
    
    lr[which(lr <= 0)] = 0
    
    df = df.full - df.reduce
    pval = 1 - pchisq(lr, df = df)
    
    qval = p.adjust(pval, method = "BH")
    
    if (length(Beta) == 1) 
        logFC <- as.vector(logFC)
    
    table = data.frame(logFC = logFC, lr = lr, pvalue = pval, qvalue = qval)
    
    
    list(table = table, dispersion = phi.hat, log.dispersion = log(phi.hat), design.full = X.full, 
         design.reduce = design.reduce, Beta.full = Beta.full, mean.full = U.full, 
         Beta.reduce = Beta.reduce, mean.reduce = U.reduce, m0 = m0, sigma = sigma)
    
    }


glmfit.full <- function(NanoStringData, design.full) {
    
    
    c = positiveFactor(NanoStringData)
    d = housekeepingFactor(NanoStringData)
    k = c * d
    lamda_i = negativeFactor(NanoStringData)
    Y = exprs(NanoStringData)
    Y_n = sweep(Y, 2, lamda_i, FUN = "-")
    Y_nph = sweep(Y_n, 2, k, FUN = "/")
    Y_nph[Y_nph <= 0] = 0.1
    
    
    
    
    nsamples = ncol(Y)
    ngenes = nrow(Y)
    nbeta = ncol(design.full)  # number of full parameters (Beta)
    
    # Beta matrix from linear model, starting value for Betas in optim
    Blm = matrix(NA, ngenes, nbeta)
    
    for (i in 1:ngenes) {
        
        model = lm(log(Y_nph[i, ]) ~ 0 + design.full)
        Blm[i, ] = model$coefficients
        
    }
    
    U = exp(Blm %*% t(design.full))
    V = sweep(U, 2, k, FUN = "*")
    
    phi.g = est.dispersion(Y, Y_nph, lamda_i, c, d)$phi
    
    
    ii = rowMins(Y) > max(negativeControl(NanoStringData))
    
    
    l = length(which(ii == TRUE))
    if (l > 0) {
        
        phi.g0 = phi.g[ii]
        lphi.g0 = log(phi.g0)
        m0 = median(lphi.g0, na.rm = TRUE)
        sigma2.mar = (IQR(lphi.g0, na.rm = TRUE)/1.349)^2
        # Here we borrow the idea to compute the base sigma for DSS The function
        # compute.baseSigma borrow the idea from Hao Wu's function
        # compute.baseSigma.nontrend in DSS Package
        sigma2.base = compute.baseSigma(exp(m0), Y[ii, ], V[ii, ], nsamples)
        sigma = sqrt(max(sigma2.mar - sigma2.base, 0.01))
    } else {
        cat("There is no data satisied that min of endo great than max 
            of negative control ", "\n")
        m0 = -2
        sigma = 1
        lphi.g0 = 10
    }
    
    
    max.phi = max(lphi.g0, 10, na.rm = TRUE)
    max.mean = max(rowMeans(Y_nph))
    
    
    
    
    get.phi <- function(dat) {
        y = dat[1:nsamples]
        Ey = dat[nsamples + (1:nsamples)]
        
        obj = function(phi) {
            alpha = 1/phi
            tmp1 = 1/(1 + Ey * phi)
            tmp2 = 1 - tmp1
            tmp2[tmp2==0] = 1e-08
            
            item1 = function(yy) {
                y_gi = yy[1]
                lamda_gi = yy[2]
                tmp2_gi = yy[3]
                t = c(0:y_gi)
                com = matrix(700, length(t), 1)
                
                
                tmp33.t = exp(rowMins(cbind(lgamma(t + alpha) + (y_gi - t) * log(lamda_gi) + 
                                                t * log(tmp2_gi) - lfactorial(t) - lfactorial(y_gi - t), com)))
                tmp33.tt = log(max(sum(tmp33.t), 1e-08))
                
            }
            tmp3 = apply(cbind(matrix(y, ncol = 1), matrix(lamda_i, ncol = 1), matrix(tmp2, 
                                                                                      ncol = 1)), 1, item1)
            
            
            -(sum(tmp3) - nsamples * lgamma(alpha) + alpha * sum(log(tmp1)) - ((log(phi) - 
                                                                                    m0)^2)/(2 * (sigma^2)) - log(sigma) - sum(lamda_i))
        }
        return(optimize(obj, interval = c(0.005, max.phi))$minimum)
    }
    
    get.beta.full <- function(dat) {
        
        n = nsamples
        y = dat[1:n]
        phi = dat[n + 1]
        Bstart = dat[(n + 2):(n + 1 + nbeta)]
        
        obj = function(beta) {
            
            
            alpha = 1/phi
            xb = beta %*% t(design.full)
            xb[xb > 700] = 700       ## control upper band for exp operation
            tmp1 = 1/(1 + exp(xb) * k * phi)
            tmp2 = 1 - tmp1
            tmp2[tmp2==0] = 1e-08
            
            
            item1 = function(yy) {
                
                
                y_gi = yy[1]
                lamda_gi = yy[2]
                tmp2_gi = yy[3]
                t = c(0:y_gi)
                com = matrix(700, length(t), 1)
                
                
                tmp33.t = exp(rowMins(cbind(lgamma(t + alpha) + (y_gi - t) * log(lamda_gi) + 
                                                t * log(tmp2_gi) - lfactorial(t) - lfactorial(y_gi - t), com)))
                tmp33.tt = log(max(sum(tmp33.t), 1e-08))
            }
            
            tmp3 = apply(cbind(matrix(y, ncol = 1), matrix(lamda_i, ncol = 1), matrix(tmp2, 
                                                                                      ncol = 1)), 1, item1)
            
            
            
            -(sum(tmp3) - n * lgamma(alpha) + alpha * sum(log(tmp1)) - ((log(phi) - 
                                                                             m0)^2)/(2 * (sigma^2)) - log(sigma) - sum(lamda_i))
        }
        
        
        return(optim(Bstart, obj)$par)
    }
    
    
    
    id = c(1:ngenes)
    Beta.full = matrix(0, ngenes, nbeta)
    phi.full = rep(0, ngenes)
    
    phi.s = apply(cbind(matrix(Y, ncol = nsamples), matrix(V, ncol = nsamples)), 
                  1, get.phi)
    B.s = Blm
    Y.t = Y
    
    con11 = 1
    con21 = 1
    
    j = 0
    while ((con11 >= 0.5 | con21 >= 0.001) & j < 50) {
        j = j + 1
        Beta = apply(cbind(matrix(Y.t, ncol = nsamples), matrix(phi.s, ncol = 1), 
                           matrix(B.s, ncol = nbeta)), 1, get.beta.full)
        
        xb = t(design.full %*% Beta)
        xb[xb > 700] = 700         ## control the upper band of exp operation
        U.t = exp(xb)
        V.t = sweep(U.t, 2, k, FUN = "*")
        
        phi.t = apply(cbind(matrix(Y.t, ncol = nsamples), matrix(V.t, ncol = nsamples)), 
                      1, get.phi)
        con1 = rowMaxs(abs((B.s - t(Beta))/t(Beta)))
        con2 = abs((phi.s - phi.t)/phi.s)
        con11 = max(con1)
        con21 = max(con2)
        
        
        idx = which(con1 < 0.5 & con2 < 0.001)
        
        if (!length(idx) == 0) {
            Beta.full[id[idx], ] = t(Beta)[idx, ]
            phi.full[id[idx]] = phi.t[idx]
        }
        
        phi.s = phi.t
        B.s = t(Beta)
        
        if (!length(idx) == 0 & !length(idx) == length(id)) {
            Y.t = Y.t[-idx, ]
            phi.s = phi.s[-idx]
            B.s = B.s[-idx, ]
            id = id[-idx]
        }
        
        # print(j)
        
    }
    
    
    if (j == 50 & !length(idx) == length(id)) {
        
        if(length(idx) == 0) {
            Beta.full[id, ] = t(Beta)
            phi.full[id] = phi.t
        } else{
            Beta.full[id, ] = t(Beta)[-idx,]
            phi.full[id] = phi.t[-idx]
        }
        
        
    }
    
    U.full = exp(Beta.full %*% t(design.full))
    V.full = sweep(U.full, 2, k, FUN = "*")
    
    
    eta = log(phi.full)
    
    
    list(Beta.full = Beta.full, design = design.full, dispersion = phi.full, log.dispersion = eta, 
         m0 = m0, sigma = sigma, df.full = nbeta, mean.full = U.full, nineration = j)
    
    
}



glmfit.reduce <- function(NanoStringData, design.reduce, m0, sigma, phi) {
    
    c = positiveFactor(NanoStringData)
    d = housekeepingFactor(NanoStringData)
    k = c * d
    lamda_i = negativeFactor(NanoStringData)
    Y = exprs(NanoStringData)
    Y_n = sweep(Y, 2, lamda_i, FUN = "-")
    Y_nph = sweep(Y_n, 2, k, FUN = "/")
    Y_nph[Y_nph <= 0] = 0.1
    
    
    
    nsamples = ncol(Y)
    ngenes = nrow(Y)
    nbeta = ncol(design.reduce)  # number of parameters (Beta)
    
    # Beta matrix from linear model, starting value for Betas in optim
    Blm = matrix(NA, ngenes, nbeta)
    
    for (i in 1:ngenes) {
        
        model = lm(log(Y_nph[i, ]) ~ 0 + design.reduce)
        Blm[i, ] = model$coefficients
        
    }
    
    get.beta.reduce <- function(dat) {
        
        n = nsamples
        y = dat[1:n]
        phi = dat[n + 1]
        Bstart = dat[(n + 2):(n + 1 + nbeta)]
        
        obj = function(beta) {
            
            
            alpha = 1/phi
            xb = beta %*% t(design.reduce)
            xb[xb > 700] = 700        ## control upper band for exp operation
            tmp1 = 1/(1 + exp(xb) * k * phi)
            tmp2 = 1 - tmp1
            tmp2[tmp2==0] = 1e-08
            
            
            item1 = function(yy) {
                
                
                y_gi = yy[1]
                lamda_gi = yy[2]
                tmp2_gi = yy[3]
                t = c(0:y_gi)
                com = matrix(700, length(t), 1)
                
                
                tmp33.t = exp(rowMins(cbind(lgamma(t + alpha) + (y_gi - t) * log(lamda_gi) + 
                                                t * log(tmp2_gi) - lfactorial(t) - lfactorial(y_gi - t), com)))
                tmp33.tt = log(max(sum(tmp33.t), 1e-08))
            }
            
            tmp3 = apply(cbind(matrix(y, ncol = 1), matrix(lamda_i, ncol = 1), matrix(tmp2, 
                                                                                      ncol = 1)), 1, item1)
            
            
            -(sum(tmp3) - n * lgamma(alpha) + alpha * sum(log(tmp1)) - ((log(phi) - 
                                                                             m0)^2)/(2 * (sigma^2)) - log(sigma) - sum(lamda_i))
        }
        
        
        return(optim(Bstart, obj)$par)
    }
    
    
    Beta.reduce = apply(cbind(matrix(Y, ncol = nsamples), matrix(phi, ncol = 1), 
                              matrix(Blm, ncol = nbeta)), 1, get.beta.reduce)
    U.reduce = exp(t(design.reduce %*% Beta.reduce))
    V.reduce = sweep(U.reduce, 2, k, FUN = "*")
    
    
    list(Beta.reduce = t(Beta.reduce), mean.reduce = U.reduce, dispersion = phi, 
         df.reduce = nbeta)
}


glmfit.OneGroup <- function(NanoStringData, m0, sigma, phi) {
    
    c = positiveFactor(NanoStringData)
    d = housekeepingFactor(NanoStringData)
    k = c * d
    lamda_i = negativeFactor(NanoStringData)
    Y = exprs(NanoStringData)
    Y_n = sweep(Y, 2, lamda_i, FUN = "-")
    Y_nph = sweep(Y_n, 2, k, FUN = "/")
    Y_nph[Y_nph <= 0] = 0.1
    
    
    
    n = ncol(Y)
    max.mean = max(rowMeans(Y_nph))
    
    get.mu <- function(dat) {
        
        y = dat[1:n]
        phi = dat[n + 1]
        
        obj = function(mu) {
            
            alpha = 1/phi
            tmp1 = 1/(1 + mu * k * phi)
            tmp2 = 1 - tmp1
            tmp2[tmp2==0] = 1e-08
            
            
            item1 = function(yy) {
                y_gi = yy[1]
                lamda_gi = yy[2]
                tmp2_gi = yy[3]
                t = c(0:y_gi)
                com = matrix(700, length(t), 1)
                
                
                tmp33.t = exp(rowMins(cbind(lgamma(t + alpha) + (y_gi - t) * log(lamda_gi) + 
                                                t * log(tmp2_gi) - lfactorial(t) - lfactorial(y_gi - t), com)))
                tmp33.tt = log(max(sum(tmp33.t), 1e-08))
            }
            
            tmp3 = apply(cbind(matrix(y, ncol = 1), matrix(lamda_i, ncol = 1), matrix(tmp2, 
                                                                                      ncol = 1)), 1, item1)
            
            
            -(sum(tmp3) - n * lgamma(alpha) + alpha * sum(log(tmp1)) - ((log(phi) - 
                                                                             m0)^2)/(2 * (sigma^2)) - log(sigma) - sum(lamda_i))
        }
        
        
        return(optimize(obj, interval = c(0.1, max.mean))$minimum)
    }
    
    mu = apply(cbind(matrix(Y, ncol = n), matrix(phi, ncol = 1)), 1, get.mu)
    
    Beta.reduce = log(mu)
    U.reduce = matrix(rep(mu, n), ncol = n)
    V.reduce = sweep(U.reduce, 2, k, FUN = "*")
    eta = log(phi)
    
    list(Beta.reduce = Beta.reduce, mean.reduce = U.reduce, dispersion = phi, df.reduce = 1)
} 

%\VignetteIndexEntry{NanoStringDiff Vignette}
%\VignettePackage{NanoStringDiff}
%\VignetteKeyword{NanoString}
%\VignetteKeyword{Differential Expression}

\documentclass[12pt]{article}

\usepackage{float}
\usepackage{Sweave}

\usepackage{amssymb}


\RequirePackage{Bioconductor}
\AtBeginDocument{\bibliographystyle{unsrturl}}

\renewcommand{\baselinestretch}{1.3}


\SweaveOpts{keep.source=TRUE,eps=FALSE,include=TRUE,width=4,height=4}



\author{Hong Wang$^{1}$, Chi Wang$^{2,3}$\footnote{to whom correspondence 
should be addressed} \\[1em] 
\small{$^{1}$Department of Statistics , University of Kentucky,Lexington, KY;}\\ 
\small{$^{2}$Markey Cancer Center, University of Kentucky, Lexington, KY ;}\\ 
\small{$^{3}$Department of Biostatistics, University of Kentucky, 
Lexington, KY;}\\ 
\small{\texttt{hong.wang@uky.edu}}}



\title{\textsf{\textbf{The NanoStringDiff package}}}

%\bibliographystyle{abbrv}

\begin{document}
\SweaveOpts{concordance=TRUE}
\maketitle

\begin{abstract}
This vignette introduces the use of the Bioconductor package 
NanoStringDiff, which is designed for differential analysis based on 
NanoString nCouner Data. NanoStringDiff considers a generalized linear 
model of the negative binomial family to characterize count data and
allows for multi-factor design. Data normalization is incorporated in
the model framework by including data normalization parameters estimated 
from positive controls, negative controls and housekeeping genes embedded 
in the nCounter system. Present method propose an empirical Bayes shrinkage 
approach to estimate the dispersion parameter and a likelihood ratio test 
to identify differential expression genes.



\end{abstract}


\newpage

\tableofcontents

\newpage


\section{Citation}
The package {\tt NanoStringDiff} implements statistical methods from the 
following publication. If you use {\tt NanoStringDiff} in the published 
research, please cite: \\
Hong Wang, Craig Horbinski, Hao Wu, Yinxing Liu,Arnold J. Stromberg 
and Chi Wang: A Negative Binomial Model-Based Method for Differential 
Expression Analysis Based on NanoString nCounter Data.(Manuscript)

\section{Quick Start}
This section show the most basic steps for a differential expression analysis 
for NanoString nCounter Data:
\begin{enumerate}
\item Create a {\tt NanoStringSet} object using {\tt createNanoStingSet} 
      or {\tt createNanoStringSetFromCsv }(see examples in the Data Input section). 
      In this section we use {\tt NanoStringData} directly, which is an object 
      of {\tt NanoStringSet} contained in the package. 
\item Make a design matrix to describe treatment conditions. 
\item Estimate norlamization factors including positive size factors, 
      housekeeping size factors and background noise using 
      {\tt estNormalizationFactors}
\item Perform a likelihood ratio test using {\tt glm.LRT}. 
\end{enumerate}


<<quick start , eval=FALSE >>=
library("Biobase")
library("NanoStringDiff")
data(NanoStringData)
pheno=pData(NanoStringData)
design.full=model.matrix(~0+pheno$group)
NanoStringData=estNormalizationFactors(NanoStringData)
result=glm.LRT(NanoStringData,design.full,contrast=c(1,-1))
@

Here, the data NanoStringData contained in the package is an animal data,
we called {\tt MoriData}\cite{mori2014hippo}. Mori et al tried to study the
possible reasons responsible for the widespread miRNA repression observed 
in cancer, global microRNA expression in mouse liver normal tissues and 
liver tumors induced by deletion of Nf2 (merlin) profiled by nCounter 
Mouse miRNA Expression Assays. Expressions of 599 miRNAs were profiled 
for two replicates in each group. 



\section{Data Input}
NanoStringDiff works on matrix of integer read counts from NanoString 
nCounter analyzer with endogenes, positive control genes, negative 
control genes and housekeeping control genes. For the matrix, rows 
corresponding to genes and columns to independent samples or replicates. 
The counts represent the total number of reads aligning to each gene
(or other genomic locus).

The count values must be raw counts of NanoString nCounter data, since 
data normalization is incorporated in the model framework by including 
data normalization parameters estimated from positive controls, negative 
controls and housekeeping genes using function {\tt estNormalizationFactors}. 
Hence, please do not supply normalized counts.

There must be have six positive control genes order by different 
concentrations from high to low, since NanoString nCounter analyzer 
provide six different samll positive controls with six different 
concentrations in the 30 uL hybridization: 128 fM, 32 fM, 8 fM, 
2 fM, 0.5 fM, and 0.125 fM. No such restriction for negative control 
genes and housekeeping control genes.Nanostring recommends at least 
three housekeeping genes, but the more that are included, the more 
accurate the normalization will be.

\subsection{Create NanoStringSet from csv.file}
The data produced by the nCounter Digital Analyzer are exported as 
a Reporter  Code Count (RCC) file. RCC files are comma-separated 
text(.csv) files that contain the counts for each gene in a sample 
and the data for each sample hybridization are contained in a 
separate RCC file. So before you call {\tt createNanoStringSetFromCsv} 
to creat a {\tt NanoStringSet} object, you should create a csv.file 
to combine all interesting samples together, and the first three 
columns shoud be "CodeClass", "Name" ane "Accession", the counts 
data contained from the $4^{th}$ column to the last column.

Note:
\begin{enumerate}
\item The $1^{st}$ column "CodeClass" should specify the function 
of genes as "Positive", "Negative","Housekeeping" or "Endogenous".
\item Some data set have "Spikein" genes, you need delete these 
"spikein" genes or you could just leave there if you use 
{\tt createNanoStringSetFromCsv} to creat {\tt NanoStringSet} 
object(See example in the Data Input section). But NanoStringDiff 
only works with "positive", "negative", housekeeping" and "endogenous".
\end{enumerate}


The "csv.file" should looks like as following Figure:

\begin{figure}[ht]
  \centering
  \includegraphics{example.PNG}
  \caption{Example of csv.file pattern}
  \label{example}
\end{figure}




When you created a csv.file, you will specify a variable which 
points on the directory in which your csv.file is located

<<directory, eval=FALSE>>=
directory <- "/path/to/your/files/"
@ 


However, for demonstration purposes only, the following line of 
code points to the directory for the "Mori.csv" in the 
NanoStringDiff package.

<<GetDirectory>>=
directory <- system.file("extdata", package="NanoStringDiff", mustWork=TRUE)
path<-paste(directory,"Mori.csv",sep="/")
@ 


The phenotype informations of the data should be stored as data frame.
<<MakePhenotypeData>>=
designs=data.frame(group=c("Normal","Normal","Tumor","Tumor"))
designs
@


<<CsvInput>>=
library("NanoStringDiff")
NanoStringData=createNanoStringSetFromCsv(path,header=TRUE,designs)
NanoStringData
@



\subsection{Create NanoStringSet from matrix}
If you already read your positive control genes, negative control genes, 
housekeeping control genes and endogous into R session separately and 
stored as matrix, then you can use {\tt createNanoStringSet} to create a 
{\tt NanoStringSet} object.


<<MatrixInput>>=
endogenous=matrix(rpois(100,50),25,4)
colnames(endogenous)=paste("Sample",1:4)
positive=matrix(rpois(24,c(128,32,8,2,0.5,0.125)*80),6,4)
colnames(positive)=paste("Sample",1:4)
negative=matrix(rpois(32,10),8,4)
colnames(negative)=paste("Sample",1:4)
housekeeping=matrix(rpois(12,100),3,4)
colnames(housekeeping)=paste("Sample",1:4)
designs=data.frame(group=c("Control","Control","Treatment","Treatment"),
                   gender=c("Male","Female","Female","Male"),
                   age=c(20,40,39,37))
NanoStringData1=createNanoStringSet(endogenous,positive,
                                 negative,housekeeping,designs)
NanoStringData1
pData(NanoStringData1)
head(exprs(NanoStringData1))
@




\section{Differential Expression Analysis for Single Factor Experiment}


For general experiments, once normalization factors obtained using 
{\tt estNormalizationFactors}, given a design matrix and contrast,
we can proceed with testing procedures for determing differential 
expression using the generalized linear model (GLM) likelihood ratio 
test. The testing can be done by using the function {\tt glm.LRT} and
return a list with a component is table including: logFC ,lr, pvalue
and qvalue(adjust p value using the procedure of Benjamini and Hochberg).


\subsection{Two Group Comparisons}

The simplest and most common type of experimental design is two group 
comparison,like treatment group vs control group. As a brief example,
consider a simple situation with control group and  treatment group, 
each with two replicates, and the researcher wants to make comparisons
between them. Here, we still use {\tt NanoSreingData1} we created above
to demonstrate this example.


Make design matrix using {\tt model.matrix}:
<<MakeDesignMatrix1>>=
pheno=pData(NanoStringData1)
group=pheno$group
design.full=model.matrix(~0+group)
design.full
@
Note that there is no intercept column in the dasign matrix, each column is 
for each group, since we include {\tt 0+} in the model formula.


If the researcher want compare Treatment to Control, that is Treatment- Control,
the contrast vector is try to the comparison -1*Control+1*Treatment, so the 
contrast is :

<<MakeContrast>>=
contrast=c(-1,1)
@


Estimate normalization factors and return the same object with positiveFactor,
negativeFactor and housekeepingFactor slots filled or replaced:
<<GetControls>>=
NanoStringData1=estNormalizationFactors(NanoStringData1)
positiveFactor(NanoStringData1)
negativeFactor(NanoStringData1)
housekeepingFactor(NanoStringData1)
@


Generalize linear model likelihood ratio test:
<<result1>>=
result=glm.LRT(NanoStringData1,design.full,contrast=contrast)
head(result$table)
str(result)
@

Note that the text Treatment compare to Control tells you that the estimates 
of logFC is log2(Treatment/Control).










\subsection{Pairwise Comparisons}
Now consider a researcher has three treatment groups say A, B and C, 
and researcher wants to compare each groups like: B compare to A, 
C compare to A, and B compare to C.


First create an object of {\tt NanoStringSet} with pseudo data:
<<CreatePseudoData>>=
endogenous=matrix(rpois(300,50),25,12)
colnames(endogenous)=paste("Sample", 1:12)
colnames(endogenous)=paste("Sample",1:12)
positive=matrix(rpois(72,c(128,32,8,2,0.5,0.125)*80),6,12)
negative=matrix(rpois(96,10),8,12)
housekeeping=matrix(rpois(36,100),3,12)
designs=data.frame(group=c(rep("A",4),rep("B",4),rep("C",4)),
                   gender=rep(c("Male","Male","Female","Female"),3),
                   age=c(20,40,39,37,29,47,23,45,34,65,35,64))
NanoStringData2=createNanoStringSet(endogenous,positive,
                                 negative,housekeeping,designs)
NanoStringData2
pData(NanoStringData2)
@



Make design matrix only consider one experimential factor {\tt group}: 
<<MakeDesignMatrix2>>=
pheno=pData(NanoStringData2)
group=pheno$group
design.full=model.matrix(~0+group)
design.full
@

Estimate normalization factors:
<<AssignSizeFactors>>=
NanoStringData2=estNormalizationFactors(NanoStringData2)
@

B compare to A, the contrast should be (-1,1,0)
<<result2,eval=FALSE>>=
result=glm.LRT(NanoStringData2,design.full,contrast=c(-1,1,0))
@


C compare to A, the contrast should be (-1,0,1)
<<result3,eval=False>>=
result=glm.LRT(NanoStringData2,design.full,contrast=c(-1,0,1))
@


B compare to c, the contrast should be (0,1,-1)
<<result4,eval=FALSE>>=
result=glm.LRT(NanoStringData2,design.full,contrast=c(0,1,-1))
@




The other way to creat a design matrix in R is to include an intercept 
term to represents the base level of the factor. We just omitte 
the {\tt 0+} in the model formula when we create design matrix using 
model.matrix function:

<<MakeDesignMatrix3>>=
design.full=model.matrix(~group)
design.full
@

In this sitution, the first coefficient measure the log scaler of baseline 
mean expression level in group A, and the second and  third column are now 
relative to the baseline, not represent the mean expression level in 
group B and C. So, if we want to compare B to A, now that is equivalent 
to test the $2^{nd}$ coefficient equal to 0.

<<result5,eval=False>>=
result=glm.LRT(NanoStringData2,design.full,Beta=2)
@

Beta=3 means compare C to A:

<<result6,eval=False>>=
result=glm.LRT(NanoStringData2,design.full,Beta=3)
@






\subsection{Multigroup Comparisons}

NanoStringDiff can do multigroup comparisons, for example if we want 
compare group A to the average of group B and C, 

<<MakeDesignMatrix4,eval=FALSE>>=
design.full=model.matrix(~0+group)
result=glm.LRT(NanoStringData2,design.full,contrast=c(1,-1/2,-1/2))
@



\section{Differential Expression Analysis for Multifactor Experiment}

In this section, we still use {\tt NanoStringData2} in examples, but the above 
examples all cotnsider single factor {\tt treatment condition} , now we include 
the other experiment factor {\tt gender} to consider multifactor problems.


\subsection{Nested Interactions}
First, we form the design matrix use factors {\tt group} and interaction 
between {\tt group} and {\tt gender}:
<<MakeDesignMatrix5>>=
design=pData(NanoStringData2)[,c("group","gender")]
group=design$group
gender=design$gender
design.full=model.matrix(~group+group:gender)
design.full
@

Here, we consider the mean expression level of female in group A as the 
baseline expression level, if we want to test the effect of gender in 
group A, we can use Beta=4,
<<result7,eval=False>>=
result=glm.LRT(NanoStringData2,design.full,Beta=4)
@


Compare treatment group B to A ignore the effect the gender, 
<<result8,eval=False>>=
result=glm.LRT(NanoStringData2,design.full,Beta=2)
@


Consider the interaction effect between gender and group B compare to A:
<<result9,eval=False>>=
result=glm.LRT(NanoStringData2,design.full,Beta=c(2,5))
@


\subsection{All Interactions}
We also can form a design matrix consider all interactions:
<<MakeDesignMatrix6>>=
design.full=model.matrix(~group+gender+group:gender)
@

Which is equivalent to:
<<MakeDesignMatrix7>>=
design.full=model.matrix(~group*gender)
design.full
@


Test the gender effect in Treatment group B:
<<result10,eval=FALSE>>=
result=glm.LRT(NanoStringData2,design.full,Beta=4:5)
@



\section{Session Info}

<<sessionInfo, results=tex, print=TRUE, eval=TRUE>>=
toLatex(sessionInfo())
@

\bibliography{reference}


\end{document}

\name{positiveFactor}
\alias{positiveFactor,NanoStringSet-method}
\alias{positiveFactor<-,NanoStringSet,numeric-method}
\alias{positiveFactor}
\alias{positiveFactor<-}
\title{
Acessor functions for the 'positiveFactor' slot in a
  NanoStringSet object.
}
\description{
  Positive size factors can be used to adjust all platform associated 
  sources of variation.

}
\usage{
\S4method{positiveFactor}{NanoStringSet}(object)
\S4method{positiveFactor}{NanoStringSet,numeric}(object) <- value
}

\arguments{

 \item{object}{A NanoStringSet object.}
 \item{value}{A vector of positive size factors.}
}

\value{A vector contain positive size factors}

\details{
 The observed counts including negative control genes and housekeeping control
 genes might be effect by some experimental factors like hybridization and 
 binding efficiency. In order to get the true rate of gene expression, 
 these variations must be normalized. Positive size factors can normalize 
 this kind of variation.
}
\author{
   Hong Wang <hong.wang@uky.edu>
   chi wang <chi.wang@uky.edu>
}

\seealso{
   positiveControl
}
\examples{
data(NanoStringData)
## obtain positive factors
positiveFactor(NanoStringData)

## assign a vector
n=ncol(exprs(NanoStringData))
positiveFactor(NanoStringData)=rep(1,n)
}

\name{negativeControl}
\alias{negativeControl,NanoStringSet-method}
\alias{negativeControl<-,NanoStringSet,matrix-method}
\alias{negativeControl}
\alias{negativeControl<-}
\title{
Acessor functions for the 'negativeControl' slot in a
  NanoStringSet object.
}
\description{
Negative control genes are provided by nCounter Analyzer which can be used to 
estimate background noise for each sample.
}
\usage{
\S4method{negativeControl}{NanoStringSet}(object)
\S4method{negativeControl}{NanoStringSet,matrix}(object) <- value
}

\arguments{

 \item{object}{A NanoStringSet object.}
 \item{value}{A matrix with negative control genes.}
}

\value{A matrix contain negative control genes}

\details{
  Each code set in the nCounter Analyzer includes several negatives 
  control genes for which no tranCounterript is expected to be present.
  We use these spike-in negative control genes to estimate
  background noise for each sample.
}
\author{
   Hong Wang <hong.wang@uky.edu>
   chi wang <chi.wang@uky.edu>
}

\seealso{
   negativeFactor
}
\examples{
data(NanoStringData)
## obtain negative control genes
negativeControl(NanoStringData)

## assign a matrix
n=ncol(exprs(NanoStringData))
r=nrow(negativeControl(NanoStringData))
negative=matrix(rpois(r*n,10),ncol=n)
negativeControl(NanoStringData)=negative
}

\name{housekeepingControl}
\alias{housekeepingControl,NanoStringSet-method}
\alias{housekeepingControl<-,NanoStringSet,matrix-method}
\alias{housekeepingControl}
\alias{housekeepingControl<-}
\title{
Acessor functions for the 'housekeepingControl' slot in a
  NanoStringSet object.
}
\description{
 user-defined housekeeping control genes can be used to estimate housekeeping 
 factors to adjust variation caused by different sample input.
}
\usage{
\S4method{housekeepingControl}{NanoStringSet}(object)
\S4method{housekeepingControl}{NanoStringSet,matrix}(object) <- value
}

\arguments{

 \item{object}{A NanoStringSet object.}
 \item{value}{A matrix with housekeeping control genes.}
}

\value{A matrix contain housekeeping control genes}

\details{
  NanoString nCounter analyzer also contains probes for a set of 
  species-specific mRNA housekeeping(reference) genes that are not 
  spike-in the system. Nanostring recommends at least three housekeeping genes, 
  but the more that are included, the more accurate the normalization will be. 
  Housekeeping control genes are expected consistent in their expression levels.
}
\author{
   Hong Wang <hong.wang@uky.edu>
   chi wang <chi.wang@uky.edu>
}

\seealso{
   housekeepingFactor
}
\examples{
data(NanoStringData)
## obtain housekeeping control genes
housekeepingControl(NanoStringData)

## assign a matrix
n=ncol(exprs(NanoStringData))
r=nrow(housekeepingControl(NanoStringData))
housekeeping=matrix(rpois(r*n,1000),ncol=n)
housekeepingControl(NanoStringData)=housekeeping
}

\name{glm.LRT}
\alias{glm.LRT}
\alias{glm.LRT,NanoStringSet-method}

\title{
 perform gene-wise likelihood ratio test for NanoString Data
}
\description{

  The method considers a generalized linear model of the negative 
  binomial family to characterize count data and allows for multi-factor 
  design. The method propose an empirical Bayes shrinkage approach to 
  estimate the dispersion parameter and use likelihood ratio test to obtain
  p-value.
}
\usage{
   glm.LRT(NanoStringData,design.full,Beta=ncol(design.full), contrast=NULL)
}

\arguments{
  \item{NanoStringData}{An object of "NanoStringSet" class.}
  
  \item{design.full}{numeric matrix giving the design matrix for the 
                     generalized linear models under full model. must be of 
                     full column rank.}
  
  \item{Beta}{integer or character vector indicating which coefficients of the 
              linear model are to be tested equal to zero. Values must be 
              columns or column names of design. Defaults to the last 
              coefficient. Ignored if contrast is specified.}
  
  \item{contrast}{numeric vector or matrix specifying one or more contrasts 
              of the linear model coefficients to be tested equal to zero.}
}

\value{
  A list
 \item{table }{A data frame with each row corresponding to a gene. Rows are 
               sorted according to likelihood ratio test statistics. 
               The columns are:
               logFC: log fold change between two groups.
               lr: likelihood ratio test statictics.
               pvalue: p-value.
               qvalue: adjust p-value using the procedure of Benjamini 
               and Hochberg.}
               
 \item{dispersion }{a vertor of dispersion}
 
 \item{log.dispersion }{a vector of log dispersion: 
                        log.dispersion=log(dispersion)}
 
 \item{design.full}{numeric matrix giving the design matrix under full 
                    generalizedlinear model.}
                    
 \item{design.reduce}{numeric matrix giving the design matrix under reduced 
                      generalizedlinear model.}
 
 \item{Beta.full}{coefficients under full model.}
 
 \item{mean.full}{mean value under full model.}
 
 \item{Beta.reduce}{coefficients under reduced model.}
 
 \item{mean.reduce}{mean value under reduced model.}
 
 \item{m0}{hyper-parameter: mean value of the prior distribution of log 
           dispersion}
 
 \item{sigma}{hyper-parameter: standard deviation of the prior distribution of
              log dispersion}
}

\author{
   hong wang<hong.wang@uky.edu>
   chi wang <chi.wang@uky.edu>
}


\examples{
data(NanoStringData)
NanoStringData=estNormalizationFactors(NanoStringData)
group=pData(NanoStringData)
design.full=model.matrix(~0+factor(group$group))
contrast=c(1,-1)
result=glm.LRT(NanoStringData,design.full,
               Beta=ncol(design.full),contrast=contrast)
head(result$table)
}


\keyword{ models }

\name{estNormalizationFactors}
\alias{estNormalizationFactors,estNormalizationFactors-method}
\alias{estNormalizationFactors}
\title{
   estimate normalization factors,include positive size factors, background 
   noise, housekeeping size factors.
}
\description{
   This function estimates positive size factors, background noise and 
   housekeeping size factors for the input "NanoStringSet" object and 
   return the same object with positiveFactor, negativeFactor and 
   housekeepingFactor slots filled or replaced.
}
\usage{
   estNormalizationFactors(NanoStringData)
}

\arguments{
  \item{NanoStringData}{ An object of "NanoStringSet" class.}
}

\value{
  The same "NanoStringSet" object with positiveFactor, negativeFactor and 
  housekeepingFactor field filled or replaced. 
}

\author{
  hong wang <hong.wang@uky.edu>
  chi wang <chi.wang@uky.edu>
}

\examples{
data(NanoStringData)
NanoStringData=estNormalizationFactors(NanoStringData)
pf=positiveFactor(NanoStringData)
nf=negativeFactor(NanoStringData)
hf=housekeepingFactor(NanoStringData)
}

\keyword{ normalization }

\name{PlotsPositiveHousekeeping}
\alias{PlotsPositiveHousekeeping,PlotsPositiveHousekeeping-method}
\alias{PlotsPositiveHousekeeping}
\title{
   Plots of positive controls and housekeeping genes
}
\description{
   This function is used to pre-check the expressions of positive controls and housekeeping genes before data analysis. Linear regression plot of positive controls and variation analysis of housekeeping genes are available. The expressions of positive controls are supposed to be linearly related to the concentration of input sample materials, and the expressions of housekeeping genes are supposed to have relatively low variation. Nanostring recommends at least three housekeeping genes, but the more that are included, the more accurate the normalization will be. 
}
\usage{
   PlotsPositiveHousekeeping(path=path, header=TRUE)
}

\arguments{
  \item{path}{ the path of the file which the data are to be read from.}
  \item{header}{ a logical value indicating whether the file contains the names of the variables as its first line. If missing, the value is determined from the file format: header is set to TRUE if and only if the first row contains one fewer field than the number of columns.}
}


\author{
  hong wang <hong.wang@uky.edu>
  tingting zhai <tingting.zhai@uky.edu>
  chi wang <chi.wang@uky.edu>
}

\examples{
##path="/Users/NanoStringdiff-Rcode/Data/horbinski.csv"
##PlotsPositiveHousekeeping(path=path, header=TRUE)
}

\keyword{Plots, Positive Controls, Housekeeping Genes}

\name{housekeepingFactor}
\alias{housekeepingFactor,NanoStringSet-method}
\alias{housekeepingFactor<-,NanoStringSet,numeric-method}
\alias{housekeepingFactor}
\alias{housekeepingFactor<-}
\title{
Acessor functions for the 'housekeepingFactor' slot in a
  NanoStringSet object.
}
\description{
  Housekeeping size factors can be used to adjust the variance 
  caused by different sample input.

}
\usage{
\S4method{housekeepingFactor}{NanoStringSet}(object)
\S4method{housekeepingFactor}{NanoStringSet,numeric}(object) <- value
}

\arguments{

 \item{object}{A NanoStringSet object.}
 \item{value}{A vector of housekeeping size factors.}
}

\value{A vector contain housekeeping factors}

\details{
 Housekeeping gene normalization corrects for different in sample input 
 between assays,since reference genes are suppose have same expression 
 rate between samples.So the read counts from housekeeping genes, after 
 subtracting background noise and adjusting by positive size factors, 
 that are not expected to vary between samples. If there exist difference, 
 which should be caused by sample input variation.
}
\author{
   Hong Wang <hong.wang@uky.edu>
   chi wang <chi.wang@uky.edu>
}

\seealso{
   housekeepingControl
}
\examples{
data(NanoStringData)
## obtain housekeeping factors
housekeepingFactor(NanoStringData)

## assign a vector
n=ncol(exprs(NanoStringData))
housekeepingFactor(NanoStringData)=rep(1,n)
}

\name{NanoStringData}
\alias{NanoStringData}
\docType{data}
\title{
A real 'NanoStringSet' object.
}
\description{
The object is created based on Mori Data with normal and tumor groups and 2
samples in each group. The object contain 599 endogenes, 6 pisitive control,
6 negative control and 4 housekeeping control.
}
\usage{data(NanoStringData)}
\value{An object of NanoStringSet}
\examples{
data(NanoStringData)
NanoStringData
}
\keyword{datasets}

\name{negativeFactor}
\alias{negativeFactor,NanoStringSet-method}
\alias{negativeFactor<-,NanoStringSet,numeric-method}
\alias{negativeFactor}
\alias{negativeFactor<-}
\title{
Acessor functions for the 'negativeFactor' slot in a
  NanoStringSet object.
}
\description{
  Negative size factors can be used to adjust background niose for each sample.

}
\usage{
\S4method{negativeFactor}{NanoStringSet}(object)
\S4method{negativeFactor}{NanoStringSet,numeric}(object) <- value
}

\arguments{

 \item{object}{A NanoStringSet object.}
 \item{value}{A vector of background noise.}
}

\value{A vector contain background noise}

\details{
 Accurate estimation of system background is essential for DE detection 
 analysis. Each code set in the nCounter Analyzer includes several negatives 
 control genes for which no tranCounterript is expected to be present.
 We use these spike-in negative control genes to estimate background 
 noise for each sample
}
\author{
   Hong Wang <hong.wang@uky.edu>
   chi wang <chi.wang@uky.edu>
}

\seealso{
   negativeControl
}
\examples{
data(NanoStringData)
## obtain negative factors
negativeFactor(NanoStringData)

## assign a vector
n=ncol(exprs(NanoStringData))
lamda=rpois(n,10)
negativeFactor(NanoStringData)=lamda
}

\name{NanoStringSet-class}
\Rdversion{1.1}
\docType{class}
\alias{NanoStringSet-class}
\alias{NanoStringSet}
% Constructor-like function:
\alias{createNanoStringSet}
\alias{createNanoStringSetFromCsv}
\title{NanoStringSet object and constructors}
\description{
 The \code{NanoStringSet} is a s4 class used to store data from NanoString 
 nCounter analyzer. This class a subclass of \code{ExpressionSet}, with six 
 more slots: positiveControl, negativeControl, housekeepingControl, 
 positiveFactor, negativeFactor and housekeepingFactor.
 The constructor functions \code{createNanoStringSet} and 
 \code{createNanoStringSetFromCsv} create a NanoStringSet object from two types 
 of input: seperate matrix or csv files. See the vignette for examples of 
 contruction from these two input types.
 }

\usage{
createNanoStringSet(endogenous,positiveControl,negativeControl,
                 housekeepingControl,designs)

createNanoStringSetFromCsv(path, header=TRUE, designs)
}

\arguments{
  \item{endogenous}{for matrix input: a matrix of non-negative integers 
                    of endogenes}
  
  \item{positiveControl}{for matrix input: a matrix of non-negative integers of 
        positive control genes. There must have 6 positive control genes order 
        by concentrations form high to low}
  
  \item{negativeControl}{for matrix input: a matrix of non-negative integers 
       of negative control genes}
        
  \item{housekeepingControl}{for matrix input: a matrix of non-negative 
       integers of housekeeping control genes}
        
  \item{designs}{for data.frame input: phenotype data for NanoString nCounter 
       data with at least one column. Each row is one sample, that is the 
       number of rows must equal number of samples or replicates in the data.}
  
  \item{path}{path to the csv file.}
  
  \item{header}{a logical value indicating whether the file contains the 
  names of the variables as its first line. The default value is TRUE.}
  
}

\value{
     A NanoStringSet object. 
}


\section{Methods}{
  \describe{
    \item{positiveControl, positiveControl<- :}{Access and set positive 
    control genes.}
    \item{negativeControl, negativeControl<- :}{Access and set negative 
    control genes.}
    \item{housekeepingControl, housekeepingControl<- :}{Access and set 
    housekeeping 
          control genes.}
    \item{positiveFactor, positiveFactor<- :}{Access and set positive factors.}
    \item{negativeFactor, negativeFactor<- :}{Access and set negative factors.}
    \item{housekeepingFactor, housekeepingFactor<- :}{Access and set 
    housekeeping factors.}
  }
}

\author{
    hong wang <hong.wang@uky.edu>
    chi wang <chi.wang@uky.edu>
}

\seealso{
  positiveControl, negativeControl, housekeepingControl,
  positiveFactor,  negativeFactor,  housekeepingFactor
}

\examples{
endogenous=matrix(rpois(100,50),25,4)
positive=matrix(rpois(24,c(128,32,8,2,0.5,0.125)*80),6,4)
negative=matrix(rpois(32,10),8,4)
housekeeping=matrix(rpois(12,100),3,4)
designs=data.frame(group=c(0,0,1,1),gender=c("male","female","female","male"),
age=c(20,40,39,37))
NanoStringData=createNanoStringSet(endogenous,positive,negative,
                                housekeeping,designs)
NanoStringData
pData(NanoStringData)
positiveControl(NanoStringData)
head(exprs(NanoStringData))

}

\keyword{ classes }

\name{NanoStringDataNormalization}
\alias{NanoStringDataNormalization,NanoStringDataNormalization-method}
\alias{NanoStringDataNormalization}
\title{
   Normalize NanoStringData
}
\description{
   This function is used to get Normalized NanoString Data after adjusting for positive size factors, background noise and housekeeping size factors. Note that the normalized data values should only be used for data exploration / visualization purposes, e.g. drawing a heatmap. To perform differential expression analysis, we recommend users to follow the procedure described in the package vignette.
}
\usage{
   NanoStringDataNormalization(path=path, header=TRUE, designs)
}

\arguments{
  \item{path}{ the path of the file which the data are to be read from.}
  \item{header}{ a logical value indicating whether the file contains the names of the variables as its first line. If missing, the value is determined from the file format: header is set to TRUE if and only if the first row contains one fewer field than the number of columns.}
  \item{designs}{ a data frame in which the length of vector matches the column number of NanoStringData}
}


\author{
  hong wang <hong.wang@uky.edu>
  tingting zhai <tingting.zhai@uky.edu>
  chi wang <chi.wang@uky.edu>
}

\examples{
##path="/Users/NanoStringdiff-Rcode/Data/horbinski.csv"
##designs=data.frame(control=c(0,0,0,1,1,1))
##NanoStringDataNormalization(path=path, header=TRUE, designs)
}

\keyword{ normalization }

\name{NanoStringDiff-package}
\alias{NanoStringDiff-package}
\alias{NanoStringDiff}
\docType{package}
\title{
  A short title line describing what the package does
}
\description{
  A more detailed description of what the package does. A length
  of about one to five lines is recommended.
}
\details{
  This section should provide a more detailed overview of how to use the
  package, including the most important functions.
}
\author{
Your Name, email optional.

Maintainer: Your Name <your@email.com>
}
\references{
  This optional section can contain literature or other references for
  background information.
}
\keyword{ package }
\seealso{
  Optional links to other man pages
}
\examples{
  \dontrun{
     ## Optional simple examples of the most important functions
     ## These can be in \dontrun{} and \donttest{} blocks.   
  }
}

\name{positiveControl}
\alias{positiveControl,NanoStringSet-method}
\alias{positiveControl<-,NanoStringSet,matrix-method}
\alias{positiveControl}
\alias{positiveControl<-}
\title{
Acessor functions for the 'positiveControl' slot in a
  NanoStringSet object.
}
\description{
  nCounter Analyzer has positive spike-in RNA hybridization controls 
  for each sample which can be used to estimate the overall efficiency 
  of hybridization and recovery for each sample.

}
\usage{
\S4method{positiveControl}{NanoStringSet}(object)
\S4method{positiveControl}{NanoStringSet,matrix}(object) <- value
}

\arguments{

 \item{object}{A NanoStringSet object.}
 \item{value}{A matrix with six positive control genes.}
}

\value{A matrix contain positive control genes}

\details{
  Positive control genes are provided by NanoString nCounter technology.
  For each sample, nCounter provide six positive controls corresponding 
  to six different concentrations in the 30 ul hybridzation:
  128fM, 32fM, 8fM, 2fM, 0.5fM, and 0.125fM. Six positive control genes must 
  be order by concentrations from high to low.
}
\author{
   Hong Wang <hong.wang@uky.edu>
   chi wang <chi.wang@uky.edu>
}

\seealso{
   positiveFactor
}
\examples{
data(NanoStringData)
## obtain positive control genes
positiveControl(NanoStringData)

## assign a matrix
n=ncol(exprs(NanoStringData))
x=matrix(c(128,32,8,2,0.5,0.125)*80,ncol=1)
positive=matrix(rpois(6*n,x),ncol=n)
positiveControl(NanoStringData)=positive
}
